package com.qingzhou.common.web.domain;

import cn.hutool.core.annotation.Alias;
import com.mybatisflex.annotation.Column;
import com.mybatisflex.annotation.Id;
import com.mybatisflex.annotation.Table;
import com.qingzhou.common.constants.UserConstant;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.noear.solon.validation.annotation.Length;
import org.noear.solon.validation.annotation.NotBlank;
import org.noear.solon.validation.annotation.NotNull;

import java.util.Set;

/**
 * 角色 对象
 * @author xm
 */
@EqualsAndHashCode(callSuper = true)
@Data
@Table(value = "sys_role")
public class SysRole extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /** 角色ID */
    @Alias("角色序号")
    @Id
    private Long roleId;

    /** 角色名称 */
    @Alias("角色名称")
    @NotBlank(message = "角色名称不能为空")
    @Length(max = 30, message = "角色名称长度不能超过30个字符")
    private String roleName;

    /** 角色权限 */
    @Alias("角色权限")
    @NotBlank(message = "权限字符不能为空")
    @Length(max = 100, message = "权限字符长度不能超过100个字符")
    private String roleKey;

    /** 角色排序 */
    @Alias("角色排序")
    @NotNull(message = "显示顺序不能为空")
    private Integer roleSort;

    /** 数据范围（1：所有数据权限；2：自定义数据权限；3：本部门数据权限；4：本部门及以下数据权限；5：仅本人数据权限） */
    @Alias("数据范围")
    private String dataScope;

    /** 菜单树选择项是否关联显示（ 0：父子不互相关联显示 1：父子互相关联显示） */
    private boolean menuCheckStrictly;

    /** 部门树选择项是否关联显示（0：父子不互相关联显示 1：父子互相关联显示 ） */
    private boolean deptCheckStrictly;

    /** 角色状态（0正常 1停用） */
    @Alias("角色状态（0正常 1停用）")
    private String status;

    /** 用户是否存在此角色标识 默认不存在 */
    @Column(ignore = true)
    private boolean flag = false;

    /** 菜单组 */
    @Column(ignore = true)
    private Long[] menuIds;

    /** 部门组（数据权限） */
    @Column(ignore = true)
    private Long[] deptIds;

    /** 角色菜单权限 */
    @Column(ignore = true)
    private Set<String> permissions;

    public boolean isAdmin() {
        return isAdmin(this.roleId);
    }

    public static boolean isAdmin(Long roleId) {
        return roleId != null && UserConstant.ADMIN_ROLE_ID == roleId;
    }

}
